from application.model import Configurations
from application.service import BenefitService


class OnSaleItemInteractor(object):
    def __init__(self, configs, benefit_service):
        # type: (Configurations, BenefitService) -> None

        self.configs = configs
        self.logger = configs.logger
        self.benefit_service = benefit_service

    def execute(self, pos_id):
        self.benefit_service.apply_automatic_benefits_on_order(pos_id)



from httpmessageprocessorutil.automapped import auto_map


@auto_map({"system_api_key": {"name": "X-Api-Key", "location": "header", "type": str},
           "pos_id": {"name": "posId", "type": str}})
class OrderFinalizeRequest(object):
    def __init__(self, system_api_key, pos_id):
        # type: (str, str) -> None

        self.system_api_key = system_api_key
        self.pos_id = pos_id



import sysactions
from custom_sysactions import user_authorized_to_execute_action


@sysactions.action
@user_authorized_to_execute_action
def do_void_lines(pos_id, line_numbers=None, auth_data=None):
    if not line_numbers:
        return False

    model = sysactions.get_model(pos_id)
    sysactions.check_current_order(pos_id, model=model)

    if not sysactions.can_void_line(model, line_numbers):
        sysactions.show_messagebox(pos_id, "$NEED_TO_HAVE_ITEM_TO_VOID")
        return False

    pos_ot = sysactions.get_posot(model)
    order = model.find("CurrentOrder/Order")
    order_id = order.get("orderId")
    sale_lines = order.findall("SaleLine")

    pos_ot.voidLine(int(pos_id), line_numbers)
    if not auth_data:
        return True

    for line_number in line_numbers.split('|'):
        deleted_line = [x for x in sale_lines if x.get("lineNumber") == line_number and x.get("level") == "0"][0]
        part_code = deleted_line.get("partCode")
        item_id = deleted_line.get("itemId")
        pos_ot.setOrderCustomProperty("VoidAuthUser", auth_data, order_id, line_number, item_id, '0', part_code)

    return True



import cfgtools
import os

from domain import Clock, TableOrderRetriever, TableOrderSaleLine, TableOrder, Table
from old_helper import convert_from_utf_to_localtime
from report import Generator
from typing import List

from dto import \
    TableReportFilterDto, \
    TableReportDto, \
    TableReportBodyDto, \
    TableReportHeaderDto, \
    TableReportOrderDto, \
    TableReportSaleLineDto


class TableReportGenerator(Generator):

    def __init__(self, filters, clock, table_order_retriever):
        # type: (TableReportFilterDto, Clock, TableOrderRetriever) -> None

        self.filters = filters
        self.clock = clock
        self.table_order_retriever = table_order_retriever

    def generate_data(
            self
    ):
        # type: (...) -> TableReportDto

        config = cfgtools.read(os.environ["LOADERCFG"])
        show_priceless_items_config = config.find_value("PrintConfigurations.ShowPricelessItems") or 'true'
        show_priceless_items = show_priceless_items_config.lower().strip() == "true"

        table = self.table_order_retriever.get_table_order(self.filters.pos_id, self.filters.table_id)
        header = self._generate_header(table)
        body = self._generate_body(table, show_priceless_items)

        return TableReportDto(header, body)

    def _generate_header(self, table):
        # type: (Table) -> TableReportHeaderDto

        table_report_header_dto = TableReportHeaderDto()
        table_report_header_dto.table_id = table.table_id
        table_report_header_dto.table_name = table.table_name
        table_report_header_dto.service_seats = table.service_seats
        table_report_header_dto.start_date = convert_from_utf_to_localtime(table.start_date)
        table_report_header_dto.totaled_date = self.clock.get_current_datetime()
        table_report_header_dto.user_id = table.user_id
        table_report_header_dto.user_name = table.user_name
        table_report_header_dto.order_id = ",".join([str(order.order_id) for order in table.orders])

        return table_report_header_dto

    def _generate_body(self, table, show_priceless_items):
        # type: (Table, bool) -> TableReportBodyDto

        body = TableReportBodyDto()
        body.total_per_seat = table.total_per_seat
        body.total_amount = table.total_amount
        body.sub_total_amount = table.sub_total_amount
        body.service_seats = table.service_seats
        body.tip = table.tip
        body.discount_amount = table.total_discounts
        body.detailed_discounts = table.detailed_discounts
        body.detailed_payments = table.detailed_payments
        body.promotional_tenders = table.promotional_tenders
        body.detailed_tip_excluded = table.detailed_tip_excluded

        body.orders = self._fill_orders_dto(table.orders, show_priceless_items)

        return body

    def _fill_orders_dto(self, table_orders, show_priceless_items):
        # type: (List[TableOrder], bool) -> List[TableReportOrderDto]
        table_order_dto_list = []  # type: List[TableReportOrderDto]

        for table_order in table_orders:
            table_report_order_dto = TableReportOrderDto()
            table_report_order_dto.order_id = table_order.order_id
            table_report_order_dto.discount_amount = table_order.discount_amount
            table_report_order_dto.total_after_discount = table_order.total_after_discount
            table_report_order_dto.total_amount = table_order.total_amount
            table_report_order_dto.total_tender = table_order.total_tender
            table_report_order_dto.split_key = table_order.split_key
            table_report_order_dto.tip = table_order.tip

            table_report_sales_line_dto_list = self._fill_sales_line_dto(table_order.sales_line, show_priceless_items)
            table_report_order_dto.sales_line_list = table_report_sales_line_dto_list

            table_order_dto_list.append(table_report_order_dto)

        return table_order_dto_list

    def _fill_sales_line_dto(self, table_order_sales_line, show_priceless_items):
        #  type: (List[TableOrderSaleLine], bool) -> List[TableReportSaleLineDto]
        table_report_sales_line_dto_list = []

        for sale_line in table_order_sales_line:
            if sale_line.qty > 0 and (sale_line.price > 0 or show_priceless_items):
                table_report_sale_line_dto = self._convert_table_order_sale_line_to_sale_line_dto(sale_line)
                table_report_sales_line_dto_list.append(table_report_sale_line_dto)

        return table_report_sales_line_dto_list

    @staticmethod
    def _convert_table_order_sale_line_to_sale_line_dto(table_order_sale_line):
        # type: (TableOrderSaleLine) -> TableReportSaleLineDto
        table_report_sale_line_dto = TableReportSaleLineDto()

        table_report_sale_line_dto.product_name = table_order_sale_line.product_name
        table_report_sale_line_dto.qty = table_order_sale_line.qty
        table_report_sale_line_dto.default_qty = table_order_sale_line.default_qty
        table_report_sale_line_dto.price = table_order_sale_line.price
        table_report_sale_line_dto.item_type = table_order_sale_line.item_type
        table_report_sale_line_dto.seat = table_order_sale_line.seat
        table_report_sale_line_dto.level = table_order_sale_line.level
        table_report_sale_line_dto.comment = table_order_sale_line.comment
        table_report_sale_line_dto.line_number = table_order_sale_line.line_number
        table_report_sale_line_dto.service_period = table_order_sale_line.service_period

        return table_report_sale_line_dto



from abc import ABCMeta, abstractmethod
from typing import Union, List

from production.box import ProductionBox
from production.model import ProductionOrder, ProdStates


def _set_invalid_if_no_items(order):
    if not order.items:
        order.prod_state = ProdStates.INVALID


class OrderChangerProductionBox(ProductionBox):
    __metaclass__ = ABCMeta

    def order_modified(self, order):
        if not order.state:
            self.debug("{} - Order without state".format(self.name))

        self.debug("{} - Order before modification:\n{}", self.name, str(order))
        altered_order = self.change_order(order)
        if isinstance(altered_order, list) and len(altered_order) > 0:
            for index, order_temp in enumerate(altered_order):
                try:
                    self.debug("\n#region    - Order ID: {} - Order Index {}\n", order_temp.order_id, index)
                    self.debug("{} - Order Index {} after modification:\n{}", self.name, index, str(order_temp))
                    self.send_to_children(order_temp)
                finally:
                    self.debug("\n#endregion - Order ID: {} - Order Index {}\n", order_temp.order_id, index)

        elif altered_order is not None:
            _set_invalid_if_no_items(altered_order)
            self.debug("{} - Order after modification:\n{}", self.name, str(altered_order))
            self.send_to_children(altered_order)

        else:
            self.debug("{} - Order removed", self.name)

    @abstractmethod
    def change_order(self, order):
        # type: (ProductionOrder) -> Union[ProductionOrder, List[ProductionOrder]]

        raise NotImplementedError()



import json
from typing import Union

class ServicePeriod(object):
    def __init__(self, service_period_json):
        # type: (Union[str, None]) -> None

        if not service_period_json:
            service_period_json = "{}"
            self.str_json = ""
        else:
            self.str_json = service_period_json

        service_period = json.loads(service_period_json)
        self.id = service_period.get("servicePeriodId")




"""Implementation of JSONEncoder
"""
from __future__ import absolute_import
import re
from operator import itemgetter
# Do not import Decimal directly to avoid reload issues
import decimal
from .compat import unichr, binary_type, text_type, string_types, integer_types, PY3
def _import_speedups():
    try:
        from . import _speedups
        return _speedups.encode_basestring_ascii, _speedups.make_encoder
    except ImportError:
        return None, None
c_encode_basestring_ascii, c_make_encoder = _import_speedups()

from .decoder import PosInf
from .raw_json import RawJSON

ESCAPE = re.compile(r'[\x00-\x1f\\"]')
ESCAPE_ASCII = re.compile(r'([\\"]|[^\ -~])')
HAS_UTF8 = re.compile(r'[\x80-\xff]')
ESCAPE_DCT = {
    '\\': '\\\\',
    '"': '\\"',
    '\b': '\\b',
    '\f': '\\f',
    '\n': '\\n',
    '\r': '\\r',
    '\t': '\\t',
}
for i in range(0x20):
    #ESCAPE_DCT.setdefault(chr(i), '\\u{0:04x}'.format(i))
    ESCAPE_DCT.setdefault(chr(i), '\\u%04x' % (i,))

FLOAT_REPR = repr

def encode_basestring(s, _PY3=PY3, _q=u'"'):
    """Return a JSON representation of a Python string

    """
    if _PY3:
        if isinstance(s, bytes):
            s = str(s, 'utf-8')
        elif type(s) is not str:
            # convert an str subclass instance to exact str
            # raise a TypeError otherwise
            s = str.__str__(s)
    else:
        if isinstance(s, str) and HAS_UTF8.search(s) is not None:
            s = unicode(s, 'utf-8')
        elif type(s) not in (str, unicode):
            # convert an str subclass instance to exact str
            # convert a unicode subclass instance to exact unicode
            # raise a TypeError otherwise
            if isinstance(s, str):
                s = str.__str__(s)
            else:
                s = unicode.__getnewargs__(s)[0]
    def replace(match):
        return ESCAPE_DCT[match.group(0)]
    return _q + ESCAPE.sub(replace, s) + _q


def py_encode_basestring_ascii(s, _PY3=PY3):
    """Return an ASCII-only JSON representation of a Python string

    """
    if _PY3:
        if isinstance(s, bytes):
            s = str(s, 'utf-8')
        elif type(s) is not str:
            # convert an str subclass instance to exact str
            # raise a TypeError otherwise
            s = str.__str__(s)
    else:
        if isinstance(s, str) and HAS_UTF8.search(s) is not None:
            s = unicode(s, 'utf-8')
        elif type(s) not in (str, unicode):
            # convert an str subclass instance to exact str
            # convert a unicode subclass instance to exact unicode
            # raise a TypeError otherwise
            if isinstance(s, str):
                s = str.__str__(s)
            else:
                s = unicode.__getnewargs__(s)[0]
    def replace(match):
        s = match.group(0)
        try:
            return ESCAPE_DCT[s]
        except KeyError:
            n = ord(s)
            if n < 0x10000:
                #return '\\u{0:04x}'.format(n)
                return '\\u%04x' % (n,)
            else:
                # surrogate pair
                n -= 0x10000
                s1 = 0xd800 | ((n >> 10) & 0x3ff)
                s2 = 0xdc00 | (n & 0x3ff)
                #return '\\u{0:04x}\\u{1:04x}'.format(s1, s2)
                return '\\u%04x\\u%04x' % (s1, s2)
    return '"' + str(ESCAPE_ASCII.sub(replace, s)) + '"'


encode_basestring_ascii = (
    c_encode_basestring_ascii or py_encode_basestring_ascii)

class JSONEncoder(object):
    """Extensible JSON <http://json.org> encoder for Python data structures.

    Supports the following objects and types by default:

    +-------------------+---------------+
    | Python            | JSON          |
    +===================+===============+
    | dict, namedtuple  | object        |
    +-------------------+---------------+
    | list, tuple       | array         |
    +-------------------+---------------+
    | str, unicode      | string        |
    +-------------------+---------------+
    | int, long, float  | number        |
    +-------------------+---------------+
    | True              | true          |
    +-------------------+---------------+
    | False             | false         |
    +-------------------+---------------+
    | None              | null          |
    +-------------------+---------------+

    To extend this to recognize other objects, subclass and implement a
    ``.default()`` method with another method that returns a serializable
    object for ``o`` if possible, otherwise it should call the superclass
    implementation (to raise ``TypeError``).

    """
    item_separator = ', '
    key_separator = ': '

    def __init__(self, skipkeys=False, ensure_ascii=True,
                 check_circular=True, allow_nan=True, sort_keys=False,
                 indent=None, separators=None, encoding='utf-8', default=None,
                 use_decimal=True, namedtuple_as_object=True,
                 tuple_as_array=True, bigint_as_string=False,
                 item_sort_key=None, for_json=False, ignore_nan=False,
                 int_as_string_bitcount=None, iterable_as_array=False):
        """Constructor for JSONEncoder, with sensible defaults.

        If skipkeys is false, then it is a TypeError to attempt
        encoding of keys that are not str, int, long, float or None.  If
        skipkeys is True, such items are simply skipped.

        If ensure_ascii is true, the output is guaranteed to be str
        objects with all incoming unicode characters escaped.  If
        ensure_ascii is false, the output will be unicode object.

        If check_circular is true, then lists, dicts, and custom encoded
        objects will be checked for circular references during encoding to
        prevent an infinite recursion (which would cause an OverflowError).
        Otherwise, no such check takes place.

        If allow_nan is true, then NaN, Infinity, and -Infinity will be
        encoded as such.  This behavior is not JSON specification compliant,
        but is consistent with most JavaScript based encoders and decoders.
        Otherwise, it will be a ValueError to encode such floats.

        If sort_keys is true, then the output of dictionaries will be
        sorted by key; this is useful for regression tests to ensure
        that JSON serializations can be compared on a day-to-day basis.

        If indent is a string, then JSON array elements and object members
        will be pretty-printed with a newline followed by that string repeated
        for each level of nesting. ``None`` (the default) selects the most compact
        representation without any newlines. For backwards compatibility with
        versions of simplejson earlier than 2.1.0, an integer is also accepted
        and is converted to a string with that many spaces.

        If specified, separators should be an (item_separator, key_separator)
        tuple.  The default is (', ', ': ') if *indent* is ``None`` and
        (',', ': ') otherwise.  To get the most compact JSON representation,
        you should specify (',', ':') to eliminate whitespace.

        If specified, default is a function that gets called for objects
        that can't otherwise be serialized.  It should return a JSON encodable
        version of the object or raise a ``TypeError``.

        If encoding is not None, then all input strings will be
        transformed into unicode using that encoding prior to JSON-encoding.
        The default is UTF-8.

        If use_decimal is true (default: ``True``), ``decimal.Decimal`` will
        be supported directly by the encoder. For the inverse, decode JSON
        with ``parse_float=decimal.Decimal``.

        If namedtuple_as_object is true (the default), objects with
        ``_asdict()`` methods will be encoded as JSON objects.

        If tuple_as_array is true (the default), tuple (and subclasses) will
        be encoded as JSON arrays.

        If *iterable_as_array* is true (default: ``False``),
        any object not in the above table that implements ``__iter__()``
        will be encoded as a JSON array.

        If bigint_as_string is true (not the default), ints 2**53 and higher
        or lower than -2**53 will be encoded as strings. This is to avoid the
        rounding that happens in Javascript otherwise.

        If int_as_string_bitcount is a positive number (n), then int of size
        greater than or equal to 2**n or lower than or equal to -2**n will be
        encoded as strings.

        If specified, item_sort_key is a callable used to sort the items in
        each dictionary. This is useful if you want to sort items other than
        in alphabetical order by key.

        If for_json is true (not the default), objects with a ``for_json()``
        method will use the return value of that method for encoding as JSON
        instead of the object.

        If *ignore_nan* is true (default: ``False``), then out of range
        :class:`float` values (``nan``, ``inf``, ``-inf``) will be serialized
        as ``null`` in compliance with the ECMA-262 specification. If true,
        this will override *allow_nan*.

        """

        self.skipkeys = skipkeys
        self.ensure_ascii = ensure_ascii
        self.check_circular = check_circular
        self.allow_nan = allow_nan
        self.sort_keys = sort_keys
        self.use_decimal = use_decimal
        self.namedtuple_as_object = namedtuple_as_object
        self.tuple_as_array = tuple_as_array
        self.iterable_as_array = iterable_as_array
        self.bigint_as_string = bigint_as_string
        self.item_sort_key = item_sort_key
        self.for_json = for_json
        self.ignore_nan = ignore_nan
        self.int_as_string_bitcount = int_as_string_bitcount
        if indent is not None and not isinstance(indent, string_types):
            indent = indent * ' '
        self.indent = indent
        if separators is not None:
            self.item_separator, self.key_separator = separators
        elif indent is not None:
            self.item_separator = ','
        if default is not None:
            self.default = default
        self.encoding = encoding

    def default(self, o):
        """Implement this method in a subclass such that it returns
        a serializable object for ``o``, or calls the base implementation
        (to raise a ``TypeError``).

        For example, to support arbitrary iterators, you could
        implement default like this::

            def default(self, o):
                try:
                    iterable = iter(o)
                except TypeError:
                    pass
                else:
                    return list(iterable)
                return JSONEncoder.default(self, o)

        """
        raise TypeError('Object of type %s is not JSON serializable' %
                        o.__class__.__name__)

    def encode(self, o):
        """Return a JSON string representation of a Python data structure.

        >>> from simplejson import JSONEncoder
        >>> JSONEncoder().encode({"foo": ["bar", "baz"]})
        '{"foo": ["bar", "baz"]}'

        """
        # This is for extremely simple cases and benchmarks.
        if isinstance(o, binary_type):
            _encoding = self.encoding
            if (_encoding is not None and not (_encoding == 'utf-8')):
                o = text_type(o, _encoding)
        if isinstance(o, string_types):
            if self.ensure_ascii:
                return encode_basestring_ascii(o)
            else:
                return encode_basestring(o)
        # This doesn't pass the iterator directly to ''.join() because the
        # exceptions aren't as detailed.  The list call should be roughly
        # equivalent to the PySequence_Fast that ''.join() would do.
        chunks = self.iterencode(o, _one_shot=True)
        if not isinstance(chunks, (list, tuple)):
            chunks = list(chunks)
        if self.ensure_ascii:
            return ''.join(chunks)
        else:
            return u''.join(chunks)

    def iterencode(self, o, _one_shot=False):
        """Encode the given object and yield each string
        representation as available.

        For example::

            for chunk in JSONEncoder().iterencode(bigobject):
                mysocket.write(chunk)

        """
        if self.check_circular:
            markers = {}
        else:
            markers = None
        if self.ensure_ascii:
            _encoder = encode_basestring_ascii
        else:
            _encoder = encode_basestring
        if self.encoding != 'utf-8' and self.encoding is not None:
            def _encoder(o, _orig_encoder=_encoder, _encoding=self.encoding):
                if isinstance(o, binary_type):
                    o = text_type(o, _encoding)
                return _orig_encoder(o)

        def floatstr(o, allow_nan=self.allow_nan, ignore_nan=self.ignore_nan,
                _repr=FLOAT_REPR, _inf=PosInf, _neginf=-PosInf):
            # Check for specials. Note that this type of test is processor
            # and/or platform-specific, so do tests which don't depend on
            # the internals.

            if o != o:
                text = 'NaN'
            elif o == _inf:
                text = 'Infinity'
            elif o == _neginf:
                text = '-Infinity'
            else:
                if type(o) != float:
                    # See #118, do not trust custom str/repr
                    o = float(o)
                return _repr(o)

            if ignore_nan:
                text = 'null'
            elif not allow_nan:
                raise ValueError(
                    "Out of range float values are not JSON compliant: " +
                    repr(o))

            return text

        key_memo = {}
        int_as_string_bitcount = (
            53 if self.bigint_as_string else self.int_as_string_bitcount)
        if (_one_shot and c_make_encoder is not None
                and self.indent is None):
            _iterencode = c_make_encoder(
                markers, self.default, _encoder, self.indent,
                self.key_separator, self.item_separator, self.sort_keys,
                self.skipkeys, self.allow_nan, key_memo, self.use_decimal,
                self.namedtuple_as_object, self.tuple_as_array,
                int_as_string_bitcount,
                self.item_sort_key, self.encoding, self.for_json,
                self.ignore_nan, decimal.Decimal, self.iterable_as_array)
        else:
            _iterencode = _make_iterencode(
                markers, self.default, _encoder, self.indent, floatstr,
                self.key_separator, self.item_separator, self.sort_keys,
                self.skipkeys, _one_shot, self.use_decimal,
                self.namedtuple_as_object, self.tuple_as_array,
                int_as_string_bitcount,
                self.item_sort_key, self.encoding, self.for_json,
                self.iterable_as_array, Decimal=decimal.Decimal)
        try:
            return _iterencode(o, 0)
        finally:
            key_memo.clear()


class JSONEncoderForHTML(JSONEncoder):
    """An encoder that produces JSON safe to embed in HTML.

    To embed JSON content in, say, a script tag on a web page, the
    characters &, < and > should be escaped. They cannot be escaped
    with the usual entities (e.g. &amp;) because they are not expanded
    within <script> tags.

    This class also escapes the line separator and paragraph separator
    characters U+2028 and U+2029, irrespective of the ensure_ascii setting,
    as these characters are not valid in JavaScript strings (see
    http://timelessrepo.com/json-isnt-a-javascript-subset).
    """

    def encode(self, o):
        # Override JSONEncoder.encode because it has hacks for
        # performance that make things more complicated.
        chunks = self.iterencode(o, True)
        if self.ensure_ascii:
            return ''.join(chunks)
        else:
            return u''.join(chunks)

    def iterencode(self, o, _one_shot=False):
        chunks = super(JSONEncoderForHTML, self).iterencode(o, _one_shot)
        for chunk in chunks:
            chunk = chunk.replace('&', '\\u0026')
            chunk = chunk.replace('<', '\\u003c')
            chunk = chunk.replace('>', '\\u003e')

            if not self.ensure_ascii:
                chunk = chunk.replace(u'\u2028', '\\u2028')
                chunk = chunk.replace(u'\u2029', '\\u2029')

            yield chunk


def _make_iterencode(markers, _default, _encoder, _indent, _floatstr,
        _key_separator, _item_separator, _sort_keys, _skipkeys, _one_shot,
        _use_decimal, _namedtuple_as_object, _tuple_as_array,
        _int_as_string_bitcount, _item_sort_key,
        _encoding,_for_json,
        _iterable_as_array,
        ## HACK: hand-optimized bytecode; turn globals into locals
        _PY3=PY3,
        ValueError=ValueError,
        string_types=string_types,
        Decimal=None,
        dict=dict,
        float=float,
        id=id,
        integer_types=integer_types,
        isinstance=isinstance,
        list=list,
        str=str,
        tuple=tuple,
        iter=iter,
    ):
    if _use_decimal and Decimal is None:
        Decimal = decimal.Decimal
    if _item_sort_key and not callable(_item_sort_key):
        raise TypeError("item_sort_key must be None or callable")
    elif _sort_keys and not _item_sort_key:
        _item_sort_key = itemgetter(0)

    if (_int_as_string_bitcount is not None and
        (_int_as_string_bitcount <= 0 or
         not isinstance(_int_as_string_bitcount, integer_types))):
        raise TypeError("int_as_string_bitcount must be a positive integer")

    def _encode_int(value):
        skip_quoting = (
            _int_as_string_bitcount is None
            or
            _int_as_string_bitcount < 1
        )
        if type(value) not in integer_types:
            # See #118, do not trust custom str/repr
            value = int(value)
        if (
            skip_quoting or
            (-1 << _int_as_string_bitcount)
            < value <
            (1 << _int_as_string_bitcount)
        ):
            return str(value)
        return '"' + str(value) + '"'

    def _iterencode_list(lst, _current_indent_level):
        if not lst:
            yield '[]'
            return
        if markers is not None:
            markerid = id(lst)
            if markerid in markers:
                raise ValueError("Circular reference detected")
            markers[markerid] = lst
        buf = '['
        if _indent is not None:
            _current_indent_level += 1
            newline_indent = '\n' + (_indent * _current_indent_level)
            separator = _item_separator + newline_indent
            buf += newline_indent
        else:
            newline_indent = None
            separator = _item_separator
        first = True
        for value in lst:
            if first:
                first = False
            else:
                buf = separator
            if isinstance(value, string_types):
                yield buf + _encoder(value)
            elif _PY3 and isinstance(value, bytes) and _encoding is not None:
                yield buf + _encoder(value)
            elif isinstance(value, RawJSON):
                yield buf + value.encoded_json
            elif value is None:
                yield buf + 'null'
            elif value is True:
                yield buf + 'true'
            elif value is False:
                yield buf + 'false'
            elif isinstance(value, integer_types):
                yield buf + _encode_int(value)
            elif isinstance(value, float):
                yield buf + _floatstr(value)
            elif _use_decimal and isinstance(value, Decimal):
                yield buf + str(value)
            else:
                yield buf
                for_json = _for_json and getattr(value, 'for_json', None)
                if for_json and callable(for_json):
                    chunks = _iterencode(for_json(), _current_indent_level)
                elif isinstance(value, list):
                    chunks = _iterencode_list(value, _current_indent_level)
                else:
                    _asdict = _namedtuple_as_object and getattr(value, '_asdict', None)
                    if _asdict and callable(_asdict):
                        chunks = _iterencode_dict(_asdict(),
                                                  _current_indent_level)
                    elif _tuple_as_array and isinstance(value, tuple):
                        chunks = _iterencode_list(value, _current_indent_level)
                    elif isinstance(value, dict):
                        chunks = _iterencode_dict(value, _current_indent_level)
                    else:
                        chunks = _iterencode(value, _current_indent_level)
                for chunk in chunks:
                    yield chunk
        if first:
            # iterable_as_array misses the fast path at the top
            yield '[]'
        else:
            if newline_indent is not None:
                _current_indent_level -= 1
                yield '\n' + (_indent * _current_indent_level)
            yield ']'
        if markers is not None:
            del markers[markerid]

    def _stringify_key(key):
        if isinstance(key, string_types): # pragma: no cover
            pass
        elif _PY3 and isinstance(key, bytes) and _encoding is not None:
            key = str(key, _encoding)
        elif isinstance(key, float):
            key = _floatstr(key)
        elif key is True:
            key = 'true'
        elif key is False:
            key = 'false'
        elif key is None:
            key = 'null'
        elif isinstance(key, integer_types):
            if type(key) not in integer_types:
                # See #118, do not trust custom str/repr
                key = int(key)
            key = str(key)
        elif _use_decimal and isinstance(key, Decimal):
            key = str(key)
        elif _skipkeys:
            key = None
        else:
            raise TypeError('keys must be str, int, float, bool or None, '
                            'not %s' % key.__class__.__name__)
        return key

    def _iterencode_dict(dct, _current_indent_level):
        if not dct:
            yield '{}'
            return
        if markers is not None:
            markerid = id(dct)
            if markerid in markers:
                raise ValueError("Circular reference detected")
            markers[markerid] = dct
        yield '{'
        if _indent is not None:
            _current_indent_level += 1
            newline_indent = '\n' + (_indent * _current_indent_level)
            item_separator = _item_separator + newline_indent
            yield newline_indent
        else:
            newline_indent = None
            item_separator = _item_separator
        first = True
        if _PY3:
            iteritems = dct.items()
        else:
            iteritems = dct.iteritems()
        if _item_sort_key:
            items = []
            for k, v in dct.items():
                if not isinstance(k, string_types):
                    k = _stringify_key(k)
                    if k is None:
                        continue
                items.append((k, v))
            items.sort(key=_item_sort_key)
        else:
            items = iteritems
        for key, value in items:
            if not (_item_sort_key or isinstance(key, string_types)):
                key = _stringify_key(key)
                if key is None:
                    # _skipkeys must be True
                    continue
            if first:
                first = False
            else:
                yield item_separator
            yield _encoder(key)
            yield _key_separator
            if isinstance(value, string_types):
                yield _encoder(value)
            elif _PY3 and isinstance(value, bytes) and _encoding is not None:
                yield _encoder(value)
            elif isinstance(value, RawJSON):
                yield value.encoded_json
            elif value is None:
                yield 'null'
            elif value is True:
                yield 'true'
            elif value is False:
                yield 'false'
            elif isinstance(value, integer_types):
                yield _encode_int(value)
            elif isinstance(value, float):
                yield _floatstr(value)
            elif _use_decimal and isinstance(value, Decimal):
                yield str(value)
            else:
                for_json = _for_json and getattr(value, 'for_json', None)
                if for_json and callable(for_json):
                    chunks = _iterencode(for_json(), _current_indent_level)
                elif isinstance(value, list):
                    chunks = _iterencode_list(value, _current_indent_level)
                else:
                    _asdict = _namedtuple_as_object and getattr(value, '_asdict', None)
                    if _asdict and callable(_asdict):
                        chunks = _iterencode_dict(_asdict(),
                                                  _current_indent_level)
                    elif _tuple_as_array and isinstance(value, tuple):
                        chunks = _iterencode_list(value, _current_indent_level)
                    elif isinstance(value, dict):
                        chunks = _iterencode_dict(value, _current_indent_level)
                    else:
                        chunks = _iterencode(value, _current_indent_level)
                for chunk in chunks:
                    yield chunk
        if newline_indent is not None:
            _current_indent_level -= 1
            yield '\n' + (_indent * _current_indent_level)
        yield '}'
        if markers is not None:
            del markers[markerid]

    def _iterencode(o, _current_indent_level):
        if isinstance(o, string_types):
            yield _encoder(o)
        elif _PY3 and isinstance(o, bytes) and _encoding is not None:
            yield _encoder(o)
        elif isinstance(o, RawJSON):
            yield o.encoded_json
        elif o is None:
            yield 'null'
        elif o is True:
            yield 'true'
        elif o is False:
            yield 'false'
        elif isinstance(o, integer_types):
            yield _encode_int(o)
        elif isinstance(o, float):
            yield _floatstr(o)
        else:
            for_json = _for_json and getattr(o, 'for_json', None)
            if for_json and callable(for_json):
                for chunk in _iterencode(for_json(), _current_indent_level):
                    yield chunk
            elif isinstance(o, list):
                for chunk in _iterencode_list(o, _current_indent_level):
                    yield chunk
            else:
                _asdict = _namedtuple_as_object and getattr(o, '_asdict', None)
                if _asdict and callable(_asdict):
                    for chunk in _iterencode_dict(_asdict(),
                            _current_indent_level):
                        yield chunk
                elif (_tuple_as_array and isinstance(o, tuple)):
                    for chunk in _iterencode_list(o, _current_indent_level):
                        yield chunk
                elif isinstance(o, dict):
                    for chunk in _iterencode_dict(o, _current_indent_level):
                        yield chunk
                elif _use_decimal and isinstance(o, Decimal):
                    yield str(o)
                else:
                    while _iterable_as_array:
                        # Markers are not checked here because it is valid for
                        # an iterable to return self.
                        try:
                            o = iter(o)
                        except TypeError:
                            break
                        for chunk in _iterencode_list(o, _current_indent_level):
                            yield chunk
                        return
                    if markers is not None:
                        markerid = id(o)
                        if markerid in markers:
                            raise ValueError("Circular reference detected")
                        markers[markerid] = o
                    o = _default(o)
                    for chunk in _iterencode(o, _current_indent_level):
                        yield chunk
                    if markers is not None:
                        del markers[markerid]

    return _iterencode



from time import sleep

from application.model.exceptions import TimeoutException


def wait(timeout=15):
    wait_time, rate, total = 0.250, 2, 0

    while total < timeout:
        wait_time = min(wait_time, timeout - total)
        sleep(wait_time)

        wait_time *= rate
        total += wait_time

        yield None

    raise TimeoutException()



import traceback
import sys

try:
    from StringIO import StringIO
except:
    from io import StringIO


class InnerException(Exception):
    def __init__(self, inner_exception=None):
        self.inner_exception = inner_exception

        if self.inner_exception is not None:
            f = StringIO()
            exc_type, exc_value, exc_traceback = sys.exc_info()
            traceback.print_exception(exc_type, exc_value, exc_traceback, file=f)
            self.inner_stack_trace = f.getvalue()

    def __str__(self):
        s = super(InnerException, self).__str__()
        if self.inner_exception is not None:
            s += "\n"
            s += "Inner Exception:\n"
            s += self.inner_stack_trace
        return s



from _CashTransferSummaryHeaderDto import CashTransferSummaryHeaderDto  # noqa
from _CashTransferSummaryBodyDto import CashTransferSummaryBodyDto  # noqa


class CashTransferSummaryDto(object):
    def __init__(self, cash_transfer_summary_header_dto, cash_transfer_summary_body_dto):
        # type:(CashTransferSummaryHeaderDto, CashTransferSummaryBodyDto) -> ()
        self.cash_transfer_summary_header_dto = cash_transfer_summary_header_dto
        self.cash_transfer_summary_body_dto = cash_transfer_summary_body_dto



# -*- coding: utf-8 -*-
import json
from xml.etree import cElementTree as eTree

import sysactions
from sysactions import (
    show_any_dialog,
    show_messagebox
)
from posot import OrderTaker

from .. import mb_context
from actions.delivery.confirm_delivery_payment import (
    set_order_tenders,
    confirm_delivery_payment_server
)


@sysactions.action
def delivery_arrangement_payment(
        pos_id,         # type:str
        order_id,       # type:str
        external_id     # type:str
):
    # type: (...) -> None

    try:

        order_take = OrderTaker(pos_id, mb_context)
        picture = order_take.orderPicture(orderid=order_id)
        order = eTree.XML(picture).find("Order")
        delivery_man = order.find(".//OrderProperty[@key='LOGISTIC_DELIVERYMAN_DATA']").get("value")

        order_gross = float(order.attrib["totalGross"])

        order_gross = "{0:.2f}".format(order_gross)
        delivery_man = "$DELIVERY_MAN_NAME_ARRANGEMENT|{}".format(delivery_man)
        order_value = "$ORDER_DELIVERY_VALUE|{}".format(order_gross)

        dialog_data = [delivery_man, order_value, "$ORDER_DELIVERY_PAYMENT"]

        buttons = "$OK|$CANCEL"
        dialog_type = "listWithInformation"
        title = "$DELIVERY_ARRANGEMENT"
        data_content = json.dumps({"linesMessage": dialog_data, "orderGross": order_gross})

        selected_option = show_any_dialog(
                pos_id,
                dialog_type,
                dialog_data,
                title,
                "",
                buttons,
                "",
                600000,
                "NOFILTER",
                "",
                "",
                "",
                data_content,
                False)

        if selected_option is None:
            return None

        if set_order_tenders(pos_id, order_id, None, order, order_take, selected_option):
            confirm_delivery_payment_server(order_id, pos_id, external_id)
            return

        show_messagebox(pos_id, "$ERROR_ON_CONFIRM_REMOTE_ORDER_DELIVERY", timeout=720000, buttons="$OK")
    except Exception as _:
        show_messagebox(pos_id, "$ERROR_ON_CONFIRM_REMOTE_ORDER_DELIVERY", timeout=720000, buttons="$OK")



from httpmessageprocessorutil.automapped import auto_map


@auto_map({"system_api_key": {"name": "X-Api-Key", "location": "header", "type": str}})
class GetUpSellPartCodesRequest(object):
    def __init__(
            self,
            system_api_key  # type: str
    ):
        # type: (...) -> None

        self.system_api_key = system_api_key



from abc import (
    ABCMeta,
    abstractmethod
)
from typing import (
    List,
    Tuple
)


class OrderRepository(object):
    __metaclass__ = ABCMeta

    @abstractmethod
    def get_database_dir(self):
        # type: () -> str

        raise NotImplementedError()

    @abstractmethod
    def get_backup_database_dir(self):
        # type: () -> str

        raise NotImplementedError()

    @abstractmethod
    def get_orders(
            self,
            pos_id,     # type: int
            end_date    # type: str
    ):
        # type: (...) -> List[Tuple[int, str]]

        raise NotImplementedError()

    @abstractmethod
    def fill_order_backup_database(
            self,
            pos_id,         # type: int
            suffix_date,    # type: str
            orders          # type: List[int]
    ):
        # type: (...) -> None

        raise NotImplementedError()

    @abstractmethod
    def delete_orders(
            self,
            pos_id,     # type: int
            orders      # type: List[int]
    ):
        # type: (int, List[int]) -> None

        raise NotImplementedError()



from ._ItemFilterBox import ItemFilterBox


class ShowOnKitchenFilterBox(ItemFilterBox):

    def __init__(self, name, sons, product_repository, logger=None):
        super(ShowOnKitchenFilterBox, self).__init__(
            name=name,
            sons=sons,
            allowed_part_codes=None,
            forbidden_part_codes=product_repository.get_not_show_on_kitchen(),
            logger=logger
        )



#!D:\Projects\e-Deploy\pos-structure\mwsdk\windows-x86\python\python.exe
# EASY-INSTALL-ENTRY-SCRIPT: 'future==0.16.0','console_scripts','futurize'
__requires__ = 'future==0.16.0'
import re
import sys
from pkg_resources import load_entry_point

if __name__ == '__main__':
    sys.argv[0] = re.sub(r'(-script\.pyw?|\.exe)?$', '', sys.argv[0])
    sys.exit(
        load_entry_point('future==0.16.0', 'console_scripts', 'futurize')()
    )



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

"""Module: barcode.ean

:Provided barcodes: EAN-13, EAN-8, JAN
"""

from barcode.base import Barcode
from barcode.charsets import ean as _ean
from barcode.errors import *

# Python 3
try:
    reduce
except NameError:
    from functools import reduce


# EAN13 Specs (all sizes in mm)
SIZES = dict(SC0=0.27, SC1=0.297, SC2=0.33, SC3=0.363, SC4=0.396, SC5=0.445,
             SC6=0.495, SC7=0.544, SC8=0.61, SC9=0.66)


class EuropeanArticleNumber13(Barcode):
    """Initializes EAN13 object.

    :parameters:
        ean : String
            The ean number as string.
        writer : barcode.writer Instance
            The writer to render the barcode (default: SVGWriter).
    """

    name = 'EAN-13'

    digits = 12

    def __init__(self, ean, writer=None, **kwargs):
        no_checksum = False
        if "no_checksum" in kwargs.keys():
            no_checksum = kwargs["no_checksum"]
        ean = ean[:self.digits]
        if not ean.isdigit():
            raise IllegalCharacterError('EAN code can only contain numbers.')
        if len(ean) != self.digits:
            raise NumberOfDigitsError('EAN must have {0} digits, not '
                                      '{1}.'.format(self.digits, len(ean)))
        self.ean = ean
        # If no checksum 
        if no_checksum:
            # Add a thirteen char if given in parameter, otherwise pad with zero
            self.ean = '{0}{1}'.format(ean, ean[self.digits] if len(ean) > self.digits else 0)
        else:
            self.ean = '{0}{1}'.format(ean, self.calculate_checksum())
        self.writer = writer or Barcode.default_writer()

    def __unicode__(self):
        return self.ean

    __str__ = __unicode__

    def get_fullcode(self):
        return self.ean

    def calculate_checksum(self):
        """Calculates the checksum for EAN13-Code.

        :returns: The checksum for `self.ean`.
        :rtype: Integer
        """
        def sum_(x, y): return int(x) + int(y)
        evensum = reduce(sum_, self.ean[::2])
        oddsum = reduce(sum_, self.ean[1::2])
        return (10 - ((evensum + oddsum * 3) % 10)) % 10

    def build(self):
        """Builds the barcode pattern from `self.ean`.

        :returns: The pattern as string
        :rtype: String
        """
        code = _ean.EDGE[:]
        pattern = _ean.LEFT_PATTERN[int(self.ean[0])]
        for i, number in enumerate(self.ean[1:7]):
            code += _ean.CODES[pattern[i]][int(number)]
        code += _ean.MIDDLE
        for number in self.ean[7:]:
            code += _ean.CODES['C'][int(number)]
        code += _ean.EDGE
        return [code]

    def to_ascii(self):
        """Returns an ascii representation of the barcode.

        :rtype: String
        """
        code = self.build()
        for i, line in enumerate(code):
            code[i] = line.replace('1', '|').replace('0', ' ')
        return '\n'.join(code)

    def render(self, writer_options=None):
        options = dict(module_width=SIZES['SC2'])
        options.update(writer_options or {})
        return Barcode.render(self, options)


class JapanArticleNumber(EuropeanArticleNumber13):
    """Initializes JAN barcode.

    :parameters:
        jan : String
            The jan number as string.
        writer : barcode.writer Instance
            The writer to render the barcode (default: SVGWriter).
    """

    name = 'JAN'

    valid_country_codes = list(range(450, 460)) + list(range(490, 500))

    def __init__(self, jan, writer=None):
        if int(jan[:3]) not in JapanArticleNumber.valid_country_codes:
            raise WrongCountryCodeError("Country code isn't between 450-460 "
                                        "or 490-500.")
        EuropeanArticleNumber13.__init__(self, jan, writer)


class EuropeanArticleNumber8(EuropeanArticleNumber13):
    """Represents an EAN-8 barcode. See EAN13's __init__ for details.

    :parameters:
        ean : String
            The ean number as string.
        writer : barcode.writer Instance
            The writer to render the barcode (default: SVGWriter).
    """

    name = 'EAN-8'

    digits = 7

    def __init__(self, ean, writer=None):
        EuropeanArticleNumber13.__init__(self, ean, writer)

    def calculate_checksum(self):
        """Calculates the checksum for EAN8-Code.

        :returns: The checksum for `self.ean`.
        :rtype: Integer
        """
        def sum_(x, y): return int(x) + int(y)
        evensum = reduce(sum_, self.ean[::2])
        oddsum = reduce(sum_, self.ean[1::2])
        return (10 - ((evensum * 3 + oddsum) % 10)) % 10

    def build(self):
        """Builds the barcode pattern from `self.ean`.

        :returns: The pattern as string
        :rtype: String
        """
        code = _ean.EDGE[:]
        for number in self.ean[:4]:
            code += _ean.CODES['A'][int(number)]
        code += _ean.MIDDLE
        for number in self.ean[4:]:
            code += _ean.CODES['C'][int(number)]
        code += _ean.EDGE
        return [code]


# Shortcuts
EAN13 = EuropeanArticleNumber13
EAN8 = EuropeanArticleNumber8
JAN = JapanArticleNumber



from xml.etree import cElementTree as eTree


class EventData(object):

    def __init__(self, data=None, event_type=None, timestamp=None, session_id=None):
        # type: (eTree, str, str, str) -> None

        self.data = data
        self.type = event_type
        self.timestamp = timestamp
        self.session_id = session_id



from abc import ABCMeta

from report import Report


class I18nReport(Report):
    __metaclass__ = ABCMeta

    def __init__(self):
        self.i18n = None

    def set_i18n(self, i18n):
        self.i18n = i18n



class Exclusions(object):
    def __init__(self, obj):
        self.items = list(obj.get("items")) if obj.get("items") else None
        self.groups = list(obj.get("groups")) if obj.get("groups") else None



import datetime

from basereport import TableReport, ReportColumnDefinition, AlignTypes
from commons.dto import DefaultHeaderDto  # noqa


class DefaultHeaderFormatter(object):
    def __init__(self, translator):
        self.report_lines = []
        self.translator = translator
        self.translated_messages = {}

    def format_header(self, title, header_dto, pos_id):
        # type: (title, DefaultHeaderDto) -> []
        self.translated_messages = self.translator.translate_labels(["$STORE",
                                                                     "$POS",
                                                                     "$ALL",
                                                                     "$OPERATOR",
                                                                     "$PERIOD",
                                                                     "$CURRENT_DATE"
                                                                     ],
                                                                    pos_id)

        self._append_header_informations(header_dto, title)

        table_formatter = TableReport([
            ReportColumnDefinition(width=13, fill_with_char='.'),
            ReportColumnDefinition(width=25, before_text=': ')
        ])

        return table_formatter.format(self.report_lines)

    def _append_header_informations(self, header_dto, title):
        self._append_line_separator()
        self._append_title(title)
        self._append_store_pos(header_dto)
        self._append_operator(header_dto)
        self._append_period(header_dto)
        self._append_current_date()
        self._append_line_separator()

    def _append_line_separator(self):
        self.report_lines.append([ReportColumnDefinition(
            fill_with_char='='
        )])

    def _append_title(self, title):
        self.report_lines.append([ReportColumnDefinition(
            text=title,
            align=AlignTypes.CENTER
        )])

    def _append_store_pos(self, header_transfer_dto):
        title = self.translated_messages["$STORE"] + "/" + self.translated_messages["$POS"]
        pos = str(header_transfer_dto.pos) if header_transfer_dto.pos is not None else self.translated_messages["$ALL"]
        value = (str(header_transfer_dto.store) + "/" + pos)

        self.report_lines.append([title, value])

    def _append_operator(self, header_transfer_dto):
        title = self.translated_messages["$OPERATOR"]
        operator = str(header_transfer_dto.operator) if header_transfer_dto.pos is not None \
            else self.translated_messages["$ALL"]
        value = operator

        self.report_lines.append([title, value])

    def _append_period(self, header_transfer_dto):
        title = self.translated_messages["$PERIOD"]
        value = str(header_transfer_dto.period)

        self.report_lines.append([title, value])

    def _append_current_date(self):
        title = self.translated_messages["$CURRENT_DATE"]
        value = datetime.datetime.now().strftime("%d/%m/%Y %H:%M:%S")

        self.report_lines.append([title, value])



# -*- coding: utf-8 -*-

import json
import logging

from bustoken import TK_CHAT_GET_UPDATES, TK_CHAT_SEND_MESSAGE, TK_CHAT_MARK_UPDATES_RECEIVED, \
    TK_CHAT_INVALID_REQUEST, TK_CHAT_GET_LAST_MESSAGES
from chatservice import ChatService, ChatModelJsonEncoder
from chatservice.model import Event
from messagehandler import EventHandler
from msgbus import MBEasyContext, FM_PARAM, MBMessage, TK_SYS_NAK, TK_SYS_ACK
from chatservice.customexception import InvalidJsonException

logger = logging.getLogger("ChatController")


class ChatControllerEventHandler(EventHandler):
    def __init__(self, mb_context, chat_service):
        # type: (MBEasyContext, ChatService) -> None
        
        super(ChatControllerEventHandler, self).__init__(mb_context)
        self.chat_service = chat_service

    def get_handled_tokens(self):
        return [TK_CHAT_GET_UPDATES, TK_CHAT_SEND_MESSAGE, TK_CHAT_MARK_UPDATES_RECEIVED, TK_CHAT_GET_LAST_MESSAGES]

    def handle_message(self, msg):
        # type: (MBMessage) -> None
        
        try:
            logger.debug("Message received: {}; Data: {}".format(msg.token, msg.data))
            
            if msg.token == TK_CHAT_GET_UPDATES:
                messages = self.chat_service.get_updates(msg.data)
                messages_json = json.dumps(messages, encoding="utf-8", cls=ChatModelJsonEncoder)
                msg.token = TK_SYS_ACK
                self.mbcontext.MB_ReplyMessage(msg, format=FM_PARAM, data=messages_json)
                
            elif msg.token == TK_CHAT_GET_LAST_MESSAGES:
                messages = self.chat_service.get_last_messages(msg.data)
                messages_json = json.dumps(messages, encoding="utf-8", cls=ChatModelJsonEncoder)
                msg.token = TK_SYS_ACK
                self.mbcontext.MB_ReplyMessage(msg, format=FM_PARAM, data=messages_json)
                
            elif msg.token == TK_CHAT_SEND_MESSAGE:
                try:
                    self.chat_service.send_message(msg.data)
                except InvalidJsonException as ex:
                    msg.token = TK_CHAT_INVALID_REQUEST
                    self.mbcontext.MB_ReplyMessage(msg, format=FM_PARAM, data=repr(ex.message).encode("utf-8"))
                msg.token = TK_SYS_ACK
                self.mbcontext.MB_EasyReplyMessage(msg)
        except Exception as ex:
            logger.exception("Error processing message: {}".format(msg.token))
            msg.token = TK_SYS_NAK
            self.mbcontext.MB_ReplyMessage(msg, format=FM_PARAM, data=repr(ex).encode("utf-8"))

    def handle_event(self, subject, evt_type, data, msg):
        # type: (unicode, unicode, str, MBMessage) -> None
        
        logger.debug("Event received: {}; Data: {}".format(subject, data))
        
        try:
            if subject == Event.SAC_CHAT_MESSAGE_NEW:
                self.chat_service.messages_received(data)
                
            elif subject == Event.SAC_CHAT_MESSAGE_ACK:
                self.chat_service.sac_ack_messages_received(data)
        except Exception as _:
            logger.exception("Error handling event: {}".format(subject))

    def terminate_event(self):
        self.chat_service.terminate()



import io
import json

from xml.etree import cElementTree as eTree
from typing import Optional, List, Dict
from datetime import datetime, timedelta
from threading import Lock
from time import sleep

import sysactions
from sysactions import set_custom, get_user_information
from msgbus import TK_SYS_NAK, TK_POS_GETPOSLIST, MBEasyContext

from dailygoals_services.model import NotCacheGoals, Goals, GoalsCache, ItemGoals, StoreGoals, OperatorGoals
from dailygoals_services.model.repository import DailyGoalsRepository
from externals.util import LoggerUtil


logger = LoggerUtil.get_logger_name(__name__)
DATE_FORMAT = "%Y-%m-%d"


class DailyGoalsService(object):
    def __init__(self, daily_goals_repository, mb_context):
        # type: (DailyGoalsRepository, MBEasyContext) -> None

        self.repository = daily_goals_repository
        self.lock = Lock()
        self.mb_context = mb_context
        sysactions.mbcontext = mb_context

        self.pos_list = None
        self.timestamp = None
        self.daily_goals = None
        self.today_goals = None
        self.store_id = None
        self.goals_cache_filename = None
        self.to_unicode = self._set_unicode()

    @staticmethod
    def _set_unicode():
        try:
            to_unicode = unicode
        except NameError:
            to_unicode = str
        return to_unicode

    def configure(self, store_id, goals_cache_filename):
        # type: (str, str) -> None

        logger.info('[configure] Begin')
        self.store_id = store_id
        self.goals_cache_filename = goals_cache_filename

        try:
            self._get_pos_list()
            if self.update_daily_goals():
                self.update_daily_sold(get_values_from_database=True)
            else:
                for pos_id in self.pos_list:
                    set_custom(pos_id, 'DAILY_GOALS', '{}')

        except Exception as _:
            logger.exception('[configure] Exception:')

        logger.info('[configure] End')

    def _get_pos_list(self):
        # type: () -> None

        retry_cnt = 10
        while retry_cnt:
            msg = self.mb_context.MB_EasySendMessage("PosController", TK_POS_GETPOSLIST)
            if msg.token == TK_SYS_NAK:
                retry_cnt -= 1
                sleep(1)
            else:
                self.pos_list = sorted(map(int, msg.data.split("\0")))
                retry_cnt = 0

    def update_daily_goals(self):
        # type: () -> Optional[bool]

        logger.info("[update_daily_goals] Begin")
        now = datetime.now()
        cur_date = now.strftime("%d/%m/%Y")

        self.timestamp = cur_date

        try:
            items_goals, store_goals = self._load_goals_from_cache()
            if not store_goals and not items_goals:
                raise NotCacheGoals()

            item_goals = self._populate_item_goals(items_goals)
            store_goals = self._populate_store_goals(store_goals, item_goals)

            with self.lock:
                self.today_goals = GoalsCache(store_goals, item_goals)
            logger.info('[update_daily_goals] End')
            return True

        except Exception as _:
            logger.exception('[update_daily_goals] Exception. Sort issue')
            raise NotImplementedError()

    def _save_to_file(self, goals):
        # type: (GoalsCache) -> None

        with io.open(self.goals_cache_filename, 'w', encoding='utf8') as outfile:
            json_goals = json.dumps(goals, default=lambda o: o.__dict__, sort_keys=True)
            outfile.write(self.to_unicode(json_goals))

    def _load_goals_from_cache(self):
        # type: () -> (List[Dict], List[Dict])

        with open(self.goals_cache_filename) as data_file:
            logger.info("[update_daily_goals] Taking goals from cache file")

            response_json = json.load(data_file)
            store_goals_json = response_json.get("storeGoals", [])
            items_goals_json = response_json.get("itemGoals", {})
        return items_goals_json, store_goals_json

    @staticmethod
    def _populate_item_goals(items_goals_json):
        # type: (List[Dict]) -> List[ItemGoals]

        item_goals = []
        if items_goals_json:
            for item in items_goals_json:
                start_date = datetime.strptime(item["startDate"], DATE_FORMAT)
                end_date = datetime.strptime(item["endDate"], DATE_FORMAT) + timedelta(days=1)
                if start_date < datetime.now() < end_date:
                    name = item.get("name")
                    part_codes = item.get("partCodes")
                    quantity = item.get("quantity")
                    quantity_operator_goal = item.get("operatorQuantity")
                    item_goals.append(ItemGoals(name, part_codes, quantity, quantity_operator_goal, 0))
        return item_goals

    @staticmethod
    def _populate_store_goals(store_goals_json, item_goals):
        # type: (List[Dict], List[ItemGoals]) -> StoreGoals

        store_goals = None
        if store_goals_json:
            for item in store_goals_json:
                start_date = datetime.strptime(item["startDate"], DATE_FORMAT)
                end_date = datetime.strptime(item["endDate"], DATE_FORMAT) + timedelta(days=1)
                if start_date < datetime.now() < end_date:
                    total_sales = item.get("totalSales")
                    average_sale_value = item.get("averageSaleValue")
                    operator_sale_goal = item.get("operatorSaleValue")
                    store_goals = StoreGoals(total_sales, average_sale_value, operator_sale_goal, item_goals)
                    break
        return store_goals

    def update_daily_sold(self, order_picture='', operator_id="", number_of_seats=1, get_values_from_database=False):
        # type: (eTree.Element, str, int, bool) -> None

        if not self.today_goals:
            return

        now = datetime.now()
        cur_date = now.strftime("%d/%m/%Y")
        if self.timestamp != cur_date:
            self.update_daily_goals()
            get_values_from_database = True

        if self.daily_goals is None:
            get_values_from_database = True

        with self.lock:
            if order_picture:
                self.daily_goals = self._update_total_sold_from_mem(order_picture, number_of_seats)
            elif get_values_from_database:
                store_goals = self.today_goals.store_goals
                items_goals = self.today_goals.item_goals
                operators_goals = []

                initial_date = datetime.now()
                end_date = datetime.now()
                self._populate_operators_sold_values(
                    end_date,
                    initial_date,
                    store_goals,
                    items_goals,
                    operators_goals
                )
                if store_goals is not None:
                    self._populate_store_sold_values(end_date, initial_date, operator_id, store_goals)

                self.daily_goals = Goals(store_goals, operators_goals)

            goals_str = json.dumps(self.daily_goals, default=lambda o: o.__dict__, sort_keys=True)
            logger.info(goals_str)
        for pos_id in self.pos_list:
            set_custom(pos_id, 'DAILY_GOALS', goals_str)

    def _update_total_sold_from_mem(self, xml_order, number_of_seats):
        # type: (eTree.Element, int) -> Optional[Goals]

        operator_id = xml_order.get("sessionId").split("user=")[1].split(",")[0]
        daily_goals = self.daily_goals
        if daily_goals is None:
            return

        if not any(x.id == operator_id for x in daily_goals.operator_goals):
            self._add_new_operator_to_mem(daily_goals, operator_id)

        added_tickets = False
        sale_lines = xml_order.findall("SaleLine")
        for line in sale_lines:
            self._process_operator_goal(added_tickets, daily_goals, line, number_of_seats, operator_id)

        return daily_goals

    def _process_operator_goal(self, added_tickets, daily_goals, line, number_of_seats, operator_id):
        part_code = line.get("partCode")
        for operator_daily_goal in daily_goals.operator_goals:
            if operator_daily_goal.id != operator_id:
                continue

            for daily_goal_item in operator_daily_goal.item_goals:
                if part_code not in daily_goal_item.part_codes:
                    continue

                self._update_quantity_sold(line, operator_daily_goal, part_code)
                self._update_quantity_sold(line, daily_goals.store_goals, part_code)

            if line.get("unitPrice") is not None:
                operator_daily_goal.amount_sold += float(line.get("qty")) * float(line.get("unitPrice"))
                daily_goals.store_goals.sold_quantity += float(line.get("qty")) * float(line.get("unitPrice"))

            if not added_tickets:
                operator_daily_goal.tickets += number_of_seats
                added_tickets = True

    @staticmethod
    def _update_quantity_sold(line, operator_daily_goal, part_code):
        for item_goal in operator_daily_goal.item_goals:
            if part_code in item_goal.part_codes:
                item_goal.quantity_sold += float(line.get("qty"))

    def _populate_operators_sold_values(self, end_date, initial_date, store_goals, item_goals, operators_goals):
        # type: (datetime, datetime, StoreGoals, List[ItemGoals], List[OperatorGoals]) -> None

        users_tickets = sysactions.generate_report("generate_tickets_report", initial_date)
        logger.info(users_tickets)
        users_tickets_json = json.loads(users_tickets)

        for op in self._get_all_opened_users():
            operator_item_goals = []
            for item_goal in item_goals:
                operator_item_goals.append(ItemGoals(
                    item_goal.name,
                    item_goal.part_codes,
                    item_goal.quantity_goal,
                    item_goal.quantity_operator_goal,
                    0)
                )

            user_tickets = list({int(v) for k, v in users_tickets_json.items() if k.startswith(op)})
            user_tickets = 0 if len(user_tickets) == 0 else user_tickets[0]

            operator_goals = OperatorGoals(
                op,
                store_goals.operator_sale_goal if store_goals is not None else 0,
                0,
                store_goals.average_sale_goal if store_goals is not None else 0,
                operator_item_goals,
                user_tickets
            )

            operator_goals.amount_sold = self._get_current_sold_qty_items_from_database(
                operator_goals.item_goals,
                initial_date,
                end_date,
                op
            )
            operators_goals.append(operator_goals)

    def _populate_store_sold_values(self, end_date, initial_date, operator_id, store_goals):
        # type: (datetime, datetime, int, List[StoreGoals]) -> None

        store_goals.sold_quantity = self._get_current_sold_qty_items_from_database(
            store_goals.item_goals,
            initial_date,
            end_date,
            operator_id
        )

    def _add_new_operator_to_mem(self, daily_goals, operator_id):
        # type: (Goals, int) -> None

        store_goals = self.today_goals.store_goals
        items_goals = self.today_goals.item_goals
        operator_item_goals = []
        for item_goal in items_goals:
            operator_item_goals.append(
                ItemGoals(
                    item_goal.name,
                    item_goal.part_codes,
                    item_goal.quantity_goal,
                    item_goal.quantity_operator_goal,
                    0
                )
            )
        operator_goals = OperatorGoals(
            operator_id,
            store_goals.operator_sale_goal,
            0,
            store_goals.average_sale_goal,
            operator_item_goals,
            0
        )
        daily_goals.operator_goals.append(operator_goals)

    @staticmethod
    def _get_all_opened_users():
        # type: () -> List
        users_list = []

        for user in eTree.XML(get_user_information()).findall(".//user"):
            users_list.append(user.get("UserId"))

        return users_list

    def _get_current_sold_qty_items_from_database(self, items, initial_date, end_date, operator_id):
        # type: (List, datetime, datetime, int) -> float

        logger.info('[_get_current_sold_qty_items_from_database] Begin')

        total_sold = 0
        pmix_order_items = json.loads(
            sysactions.generate_report(
                "pmix_order_items",
                initial_date,
                end_date,
                self.store_id,
                operator_id
            )
        )

        for pmix_item in pmix_order_items:
            for item in items:
                for part_code in item.part_codes:
                    if int(part_code) == pmix_item['pcode']:
                        item.quantity_sold += pmix_item['quantity']
            total_sold += pmix_item['quantity'] * float(pmix_item["unit_price"])

        logger.info('[_get_current_sold_qty_items_from_database] End')

        return total_sold



# -*- coding: utf-8 -*-
import logging
from datetime import datetime, timedelta
from xml.etree import cElementTree as eTree

from sysactions import get_model, get_business_period, get_cfg, is_day_opened, has_current_order
from systools import sys_log_exception

from check_table_status import has_tab_or_tables_opened
from typing import Optional

pos_block_hour = {}
pos_datetime_to_block_pos = {}

logger = logging.getLogger("TableActions")


def pos_must_be_blocked(pos_id):
    # type: (str) -> bool

    try:
        model = get_model(pos_id)
        if not pos_need_to_be_blocked(pos_id, model):
            return False

        if has_current_order(model):
            return False

        login_mode = model.find("WorkingMode").get("usrCtrlType")
        pod_type = model.find("WorkingMode").get("podType")

        if not _pos_serves_table(pod_type, pos_id):
            return True

        if login_mode == "QSR":
            return _logged_user_has_no_open_table(pos_id)
        else:
            return _all_tables_are_closed(pos_id)

    except Exception as _:
        sys_log_exception("Error verifying if POS {} must be blocked".format(pos_id))

    return False


def pos_need_to_be_blocked(pos_id, model=None):
    # type: (str, Optional[eTree.XML]) -> bool

    try:
        if model is None:
            model = get_model(pos_id)

        if pos_id not in pos_block_hour:
            pos_block_hour[pos_id] = int(get_cfg(pos_id).key_value("hourToBlockPos", "0"))

        if pos_id in pos_datetime_to_block_pos:
            datetime_to_block_pos = pos_datetime_to_block_pos[pos_id]
        else:
            hour_to_block_pos = pos_block_hour[pos_id]
            period = get_business_period(model)

            if period == '0' or not is_day_opened(model):
                return False

            datetime_to_block_pos = datetime(int(period[:4]), int(period[4:6]), int(period[6:8]))
            datetime_to_block_pos += timedelta(days=1)
            datetime_to_block_pos += timedelta(hours=hour_to_block_pos)

        current_date = datetime.now()
        if current_date > datetime_to_block_pos:
            if pos_id in pos_datetime_to_block_pos:
                pos_datetime_to_block_pos.pop(pos_id, None)

            return True
        else:
            pos_datetime_to_block_pos[pos_id] = datetime_to_block_pos
    except Exception as _:
        sys_log_exception("Error verifying if POS {} must be blocked".format(pos_id))

    return False


def _pos_serves_table(pod_type, pos_id=None):
    # type: (str, Optional[int]) -> bool

    return pod_type not in ["FC", "TT", "DL"] and pos_id != 0


def _logged_user_has_no_open_table(pos_id):
    # type: (str) -> bool

    model = get_model(pos_id)
    period = get_business_period(model)
    operator = model.find("Operator")
    if operator is None:
        return False

    user_id = operator.get("id")
    return not has_tab_or_tables_opened(period, pos_id, user_id)


def _all_tables_are_closed(pos_id):
    # type: (str) -> bool

    model = get_model(pos_id)
    period = get_business_period(model)

    return not has_tab_or_tables_opened(period, pos_id)
